@@ -12,7 +12,6 @@ class Intersection:
 
     def __init__(self, **kwargs):
         """
-
         Args:
             **kwargs:
             -> intersection_lineLength=self.config.intersection.lineLength,
@@ -26,10 +25,8 @@ class Intersection:
     @staticmethod
     def apply_confidence_rule(c):
         """
-
         Args:
             c: detected car location
-
         Returns:
             it's score for how many has captured car locations and calculate 0 < result < 100
         """
@@ -48,22 +45,14 @@ class Intersection:
             only two position cars calculate and return such as [True, False],  [False, False]
         """
         rets = []
-        for params in frameFilters:
-            ## n = chech between a and b parameter
-            ## a start point angle
-            ## b finish point angle
-            n = int(params[0])
-            a = int(params[1])
-            b = int(params[2])
-            n = (360 + (n % 360)) % 360
-            a = (3600000 + a) % 360
-            b = (3600000 + b) % 360
-            if a < b:
-                result = a <= n <= b
-                rets.append(result)
-            else:
-                result = a <= n or n <= b
-                rets.append(result)
+        for (theta,max_theta,min_theta) in frameFilters:
+
+            theta_norm = (360 + theta) % 360
+            # If max_theta is smaller than min_theta, unnormalize theta_norm and max_theta, it provides to use formula.
+            if max_theta < min_theta:
+                max_theta+=360
+            result = max_theta >= theta_norm >= min_theta
+            rets.append(result)
 
         return rets
 
@@ -76,7 +65,6 @@ class Intersection:
             angle  = 50 and heading = 320  result {theta must be between max 270 min 30}
         Args:
             nArr: its list and include headings
-
         Returns:
             up limit and down limit for theta degrees.
         """
@@ -84,19 +72,20 @@ class Intersection:
         a = self.intersection_angle_wide
         ret = []
         for n in nArr:
-            t = 360 + n
-            x = (-(a - t)) % 360
-            y = (a + t) % 360
+            min_theta=n-a/2
+            max_theta=n+a/2
+            min_theta_norm=(min_theta+360)%360
+            max_theta_norm=(max_theta+360)%360
             # x, y hands of angle
-            ret.append([x, y])
+            ret.append([min_theta_norm, max_theta_norm])
 
         anglesArr = [item for sublist in ret for item in sublist]
         assert len(anglesArr) >= 4, "Check your Thetas"
         angles = Dict({
-            'max1Ahead': anglesArr[0],
-            'min1Ahead': anglesArr[1],
-            'max2Ahead': anglesArr[2],
-            'min2Ahead': anglesArr[3]
+            'min1Ahead': anglesArr[0],
+            'max1Ahead': anglesArr[1],
+            'min2Ahead': anglesArr[2],
+            'max2Ahead': anglesArr[3],
         })
         del anglesArr
         return angles
@@ -108,22 +97,18 @@ class Intersection:
         when we change it from the config file to specify the line length here.
         Args:
             loc: calculated coordinates
-
         Returns:
-
         """
         ops = []
         for i in range(0, len(loc), 3):
             ops.append(Distance.destination_point(lat=loc[i], lon=loc[i + 1], distance=self.intersection_lineLength,
-                                                  bearing=loc[i + 2]))
-
+                                                  theta=loc[i + 2]))
         return ops[0], ops[1]
 
     @staticmethod
     def check_line_intersection(**kwargs) -> Dict:
         """
         Calculate from two points intersected locations
-
         Args:
             **kwargs:
             -> line1StartX
@@ -134,9 +119,7 @@ class Intersection:
             -> line2StartY
             -> line2EndX
             -> line2EndY
-
         Returns:
-
         """
 
         points = Dict(kwargs)
@@ -198,11 +181,9 @@ class Intersection:
     def intersection_points_average(self, groupMatches: list, geojsonFormatFunc: Callable) -> Tuple[
         Dict, list, list, List[list]]:
         """
-
         Args:
             groupMatches: matched objects same class, same points
             geojsonFormatFunc: it's function for creating geojson format
-
         Returns:
             It calculates a single point by taking the averages of all the detected intersections.
         """
@@ -240,14 +221,14 @@ class Intersection:
             objects['avg_score'].append(float(matches.score_1))
             objects['avg_score'].append(float(matches.score_2))
 
-            total['detectedPath_1'] = matches.detectedPath_1
-            total['detectedPath_2'] = matches.detectedPath_2
+            total[i]['detectedPath_1'] = matches.detectedPath_1
+            total[i]['detectedPath_2'] = matches.detectedPath_2
 
-            total['imgUrl_1'] = matches.imgUrl_1
-            total['imgUrl_2'] = matches.imgUrl_2
+            total[i]['imgUrl_1'] = matches.imgUrl_1
+            total[i]['imgUrl_2'] = matches.imgUrl_2
 
-            total['objId_1'] = matches.objId_1
-            total['objId_2'] = matches.objId_2
+            total[i]['objId_1'] = matches.objId_1
+            total[i]['objId_2'] = matches.objId_2
             total['match_id'] = matches.match
 
             total['classname'] = matches.classname_1  # doesn't matter same classname_1 and classname_2
@@ -273,8 +254,8 @@ class Intersection:
             for c in v:
                 if c == 0:
                     counter += 1
-            if counter==len(v):
-                counter=0
+            if counter == len(v):
+                counter = 0
             if k in ["Alt_cornerB", "Alt_cornerC", "Alt_cornerD", "Alt_cornerA"]:
                 total[k] = float(sum(v) / (len(v) - counter))
             else:
@@ -282,7 +263,6 @@ class Intersection:
 
         total['confidence'] = (total['avg_score'] + confidence) * 0.5
         del objects
-
         return total, pointsMerged, matched_object_id, matched_paired
 
     def intersection_points_find(self, **kwargs):
@@ -297,7 +277,6 @@ class Intersection:
             -> start_lon2
             -> theta2
             -> type : intersect or area
-
         Returns:
             get coordinates
         """
@@ -322,6 +301,10 @@ class Intersection:
                                                         line2StartY=points.start_lon2,
                                                         line2EndX=ops2["lat"],
                                                         line2EndY=ops2["lon"])
+            if interSection['l1'] and interSection['l2']:
+                interSection['z'] = self.calc_intersect_z(points.start_lon1, points.start_lat1, points.start_alt1, points.phi1,
+                                                          points.start_lon2, points.start_lat2, points.start_alt2, points.phi2,
+                                                          interSection)
 
             return interSection, destinationPoint1, destinationPoint2
 
@@ -344,40 +327,47 @@ class Intersection:
                                                             line2EndY=ops2["lon"]
                                                             )
                 if is_intersect['l1'] and is_intersect['l2']:
-
-                    distance_between_panoroma_first_and_intersected_point = Distance.haversine(
-                        lon1=points.start_lon1, lat1=points.start_lat1,
-                        lon2=is_intersect['y'], lat2=is_intersect['x'])
-
-                    center_altA = (math.tan(
-                        math.radians(ph1)) * distance_between_panoroma_first_and_intersected_point +
-                                   float(points.start_alt1))
-
-                    distance_between_panoroma_second_and_intersected_point = Distance.haversine(
-                        lon1=points.start_lon2, lat1=points.start_lat2,
-                        lon2=is_intersect['y'], lat2=is_intersect['x'])
-
-                    center_altB = (math.tan(
-                        math.radians(ph2)) * distance_between_panoroma_second_and_intersected_point +
-                                   float(points.start_alt2))
-
-                    is_intersect['z'] = (center_altA + center_altB) / 2
+                    is_intersect['z'] = self.calc_intersect_z( points.start_lon1,points.start_lat1,points.start_alt1,ph1,
+                                                               points.start_lon2, points.start_lat2, points.start_alt2, ph2,
+                                                               is_intersect)
 
                 corners[corner_id] = is_intersect
-
             return corners
+    def calc_intersect_z(self, start_lon1,start_lat1,start_alt1,phi1,
+                               start_lon2, start_lat2, start_alt2, phi2,
+                               is_intersect
+                         ):
 
+        distance_between_panoroma_first_and_intersected_point = Distance.haversine(
+            lon1=start_lon1, lat1=start_lat1,
+            lon2=is_intersect['y'], lat2=is_intersect['x'])
+        # it changes cos range as 0-90 and altitude always increases according this formula
+        # ph1 = 180 - ph1 if ph1 > 90 else ph1
+        # ph2 = 180 - ph2 if ph2 > 90 else ph2
+
+        # https://mathinsight.org/spherical_coordinates
+        center_altA = (math.cos(
+            math.radians(phi1)) * distance_between_panoroma_first_and_intersected_point +
+                       float(start_alt1))
+
+        distance_between_panoroma_second_and_intersected_point = Distance.haversine(
+            lon1=start_lon2, lat1=start_lat2,
+            lon2=is_intersect['y'], lat2=is_intersect['x'])
+
+        center_altB = (math.cos(
+            math.radians(phi2)) * distance_between_panoroma_second_and_intersected_point +
+                       float(start_alt2))
+
+        return (center_altA + center_altB) / 2
 
 
 def intersection_float_to_decimal(lat1, lon1, lat2, lon2) -> Tuple:
     """
-
     Args:
         lat1:
         lon1:
         lat2:
         lon2:
-
     Returns:
         points convert decimal format
     """
